/* be_c_int.c
 *
 * a C backend (generates C code from the intermediate representation)
 *
 * Copyright (C) 2004 Manuel Schiller
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include "bf.h"

int gen_C_int(FILE *out, bfinstr_t *pgm)
{
	long ip = 0;

	/* write the the start... */
	fprintf(out, "#include <stdio.h>\n#include <stdlib.h>\nint main()\n");
	fprintf(out, "{\n\tint *ptr = calloc(30000, sizeof(int)), *p = ptr;\n");
	/* write the program */
	while (pgm[ip].instr != EOP) {
		switch(pgm[ip].instr) {
		case ADDI:
		     	if (pgm[ip].attr == 0) break;
			fprintf(out, "\t*p += %ld;\n", pgm[ip].attr);
			break;
		case PTRI:
		     	if (pgm[ip].attr == 0) break;
			fprintf(out, "\tp += %ld;\n", pgm[ip].attr);
			break;
		case ADD:
		     	if (pgm[ip].attr2 == 0) break;
			fprintf(out, "\t*(p + %ld) += *p * %ld;\n", \
				pgm[ip].attr, pgm[ip].attr2);
			break;
		case ZERO:
			fprintf(out, "\t*p = 0;\n");
			break;
		case LOOP:
		     	fprintf(out, "\twhile(*p) {\n");
			break;
		case POOL:
		     	fprintf(out, "\t}\n");
			break;
		case IN:
		     	fprintf(out, "\t*p = getchar();\n");
			break;
		case OUT:
		     	fprintf(out, "\tputchar(*p);\n");
			break;
		default:
			break;
		}
		ip++;
	}
	/* write the end */
	fprintf(out, "\tfree(ptr);\n\treturn 0;\n}\n\n");

	return 0;
}
